
/*****************************************************
*                 glcd.c
*
* lcd driver for KS0713 and compatible 
*
* Parallel Mode Only at this stage!!
* 
* Target: PIC24 dsPIC30 and dsPIC33 devices
*         MPLAB-C30 Compiler
*
* Author: Michael Pearce <mike@kiwacan.co.nz>
*
* Started: 10 Jan 2007
*
*
* Based oncode written by Zelea for Atmel AVR
* Found at http://nic.ath.cx/graphic-lcd/
*    
*
* This program is provided under the GPL license
*   http://www.gnu.org/copyleft/gpl.html
*
*****************************************************/


/*****************************************************
* Include glcd.h
* Modify port settings, processor type etc
* in glcd.h to suit your specific needs
*****************************************************/
#include "glcd.h"

#include <string.h>

//#include <math.h>


// Initialise port settings etc
void lcd_init_io(void)
{
 /* Data port settings */
 DATA_OUT=0;
 DATA_DIR=0xFF;

 /* Command Port initial output settings */
 RD=1;
 WR=1;   
 RS=1;   
 RESET=1;
 CS=1;
 //PS=1;
 MI=0;   /* Mode of operation 6800 [1] or 8080 [0] */


 /* Command port direction settings */
 T_RD=0;
 T_WR=0;   
 T_RS=0;   
 T_RESET=0;
 T_CS=0;  
// T_PS
 T_MI=0;  


}


// set direction to read 
static void lcd_set_port_read( void )
{
  DATA_DIR = LCD_DIR_READ;
}

// set direction to write 
static void lcd_set_port_write( void )
{
  DATA_DIR = LCD_DIR_WRITE;
}

// send cmd 
static void lcd_send_cmd( u08 cmd, u08 flag )
{
  lcd_set_port_write(  );
  CS = 0;
  cmd |= flag;
  WR=0;
  DATA_OUT = cmd;
  _delay_us(LCD_DELAY);
  WR=1;
  _delay_us(LCD_DELAY);
  CS=1;
}

// write byte 
void lcd_write_byte( u08 data )
{
  lcd_set_port_write();
  CS=0;
  RS=1;
  // write data
  DATA_OUT = data;
  WR=0;
  _delay_us(LCD_DELAY);
  WR=1;
  _delay_us(LCD_DELAY);
  RS=0;
  CS=0;
}

// read status of display 
static u08 lcd_read_status( void )
{
  u08           status;

  lcd_set_port_read(  );
  CS=0;
  RD=0;
  _delay_us(LCD_DELAY);
  RD=1;
  _delay_us(LCD_DELAY);
  status = ( DATA_IN & 0xF0 );
  CS=1;
  return status;
}

// read function 
static void lcd_read_dummy( void )
{
  lcd_set_port_read(  );
  CS=0;
  RS=1;
  // write data
  RD=0;
  _delay_us(LCD_DELAY);
  RD=1;
  _delay_us(LCD_DELAY);
  RS=0;
  CS=1;
}


// read byte 
static u08 lcd_read_byte( void )
{
  u08           data;

  lcd_set_port_read(  );
  CS=0;
  RS=1;
  // read data
  RD=0;
  
  _delay_us(LCD_DELAY);
  RD=1;
  
  _delay_us(LCD_DELAY);
  data = DATA_IN;
  RS=0;
  CS=1;
  return data;
}

void lcd_read_dram( unsigned char *dest )
{
  u08           pa, col;

  for ( pa = 0; pa < LCD_ROWS; pa++ )
  {
    // set dram pointer
    lcd_set_pos( pa, 0 );

    lcd_read_dummy(  );

    for ( col = 0; col < LCD_COLS; col++ )
    {
      dest[( pa * LCD_COLS ) + col] = lcd_read_byte(  );
    }
  }
}


void lcd_write_dram( unsigned char *source )
{
  short         pa, col;

  for ( pa = 0; pa < LCD_ROWS; pa++ )
  {
    lcd_set_pos( pa, 0 );
    for ( col = 0; col < LCD_COLS; col++ )
      lcd_write_byte( source[( pa * LCD_COLS ) + col] );
  }
}

void lcd_set_pos( u08 row, u08 col )
{
  lcd_send_cmd( LCD_CMD_SPAGE, row );
  lcd_send_cmd( LCD_CMD_COL, ( col >> 4 ) & 0x0F );
  lcd_send_cmd( 0x00, col & 0x0F );
}

void lcd_clear( void )
{
  short         pa, col;

  for ( pa = 0; pa < LCD_ROWS; pa++ )
  {
    lcd_set_pos( pa, 0 );
    for ( col = 0; col < LCD_COLS; col++ )
      lcd_write_byte( 0 );
  }
}

void lcd_reset_init( void )
{
  lcd_init_io();
  lcd_set_port_read(  );
  CS=1;RESET=1;RS=1;WR=1;RD=1;
  // reset, commad mode
  RESET=0;RS=0;

  _delay_us( LCD_DELAY );

  RESET=1; // start

  _delay_us( LCD_DELAY );

  lcd_send_cmd( LCD_CMD_RESET, 0 );

  while( lcd_read_status() & LCD_STAT_RESETB );

  lcd_send_cmd( LCD_CMD_ON, 1 );
  //lcd_send_cmd( LCD_CMD_RES, 7 );
  lcd_send_cmd( LCD_CMD_RES, 6 );

  lcd_send_cmd( LCD_CMD_SRV, 1 );
  //lcd_send_cmd( 0x00, 15 );
  lcd_send_cmd( 0x00, 5 );

  if ( 0 ) lcd_send_cmd( LCD_CMD_BIAS, 0 );
  else     lcd_send_cmd( LCD_CMD_BIAS, 1 );
  lcd_send_cmd( LCD_CMD_POWERC, 7 );
  lcd_send_cmd( LCD_CMD_SIR, 1 );
  lcd_send_cmd( 0x00, 3 );	  // Always on
  lcd_send_cmd( LCD_CMD_ADC, 0 ); // flip horizontal
  lcd_send_cmd( LCD_CMD_SHL, 0 ); // flip vertical
  lcd_send_cmd( LCD_CMD_EON, 0 );
  lcd_send_cmd( LCD_CMD_REVERSE, 0 );
  lcd_send_cmd( LCD_CMD_IDL, 0 );
}

void show_icon( unsigned char *bitmap, u08 w, u08 h, u08 x, u08 y, u08 mode )
{
  u08           off, tx, ty, pix, hb;
  unsigned short col;

  off = y & 7;
  hb = (h+7) / 8;
  for ( ty = 0; ty < hb; ty++ )
  {
    lcd_set_pos( ty + y / 8, x );
    for ( tx = 0; tx < w; tx++ )
    {
      if( off )
      {
        //col = pgm_read_byte_near( bitmap + ty * w + tx ) << 8;
        col = *(bitmap+ ty * w + tx) << 8;

	if( ty )
	  //col |= pgm_read_byte_near( bitmap + (ty - 1) * w + tx );
      col |=*( bitmap + (ty - 1) * w + tx );
	col >>= 8 - off;
	pix = col;
      }
      else
        //pix = pgm_read_byte_near( bitmap + ty * w + tx );
        pix = *( bitmap + ty * w + tx );
      switch( mode )
      {
	case LCD_PIXEL_OFF:
	  pix = ~pix;
	  break;
	case LCD_PIXEL_ON:
	  break;

	case LCD_PIXEL_INV:
	  lcd_read_dummy();
	  pix ^= lcd_read_byte();
          lcd_set_pos( ty + y / 8, x + tx);
	  break;

      }
      lcd_write_byte( pix );
    }
  }
}

u08 draw_point( u08 x, u08 y, u08 v )
{
  u08           ty = 0, tval, bit;

  if ( y >= ( LCD_ROWS * 8 ) )
    return -1;

  if ( x >= LCD_COLS )
    return -1;

  if ( y )
    ty = ( y / 8 );

  bit = y - ( ty * 8 );

  // set dram pointer
  lcd_set_pos( ty, x );

  lcd_read_dummy(  );
  tval = lcd_read_byte(  );

  // inverted
  if ( v == LCD_PIXEL_INV )
    v = ( ( ( ~tval ) >> bit ) & 1 );
  if ( v == LCD_PIXEL_ON )
    tval |= ( 1 << bit );
  else
    tval &= ~( 1 << bit );

  // set dram pointer
  lcd_set_pos( ty, x );
  lcd_write_byte( tval );
  return 0;
}


/*
 * draw_line
 * 
 * args:
 * x1    StartCol
 * y1    StartRow
 * x2    EndCol
 * y1    EndRow
 * state LCD_PIXEL_OFF/LCD_PIXEL_ON/LCD_PIXEL_INV
 * 
 */

void draw_line( u08 x1, u08 y1, u08 x2, u08 y2, u08 v )
{
  short         dx, dy, sdx, sdy, px, py, dxabs, dyabs, i;
  float         slope;

  dx = x2 - x1;
  dy = y2 - y1;
  dxabs = sdx = 0;
  if( dx > 0 )
  {
    dxabs = dx;
    sdx = 1;
  }
  else if ( dx < 0 )
  {
    dxabs = -dx;
    sdx = -1;
  }
  dyabs = sdy = 0;
  if( dy > 0 )
  {
    dyabs = dy;
    sdy = 1;
  }
  else if ( dy < 0 )
  {
    dyabs = -dy;
    sdy = -1;
  }
  if ( dxabs >= dyabs )		/* the line is more horizontal than vertical */
  {
    slope = ( float ) dy / ( float ) dx;
    for ( i = 0; i != dx; i += sdx )
    {
      px = i + x1;
      py = slope * i + y1;
      draw_point( px, py, v );
    }
  }
  else				/* the line is more vertical than horizontal */
  {
    slope = ( float ) dx / ( float ) dy;
    for ( i = 0; i != dy; i += sdy )
    {
      px = slope * i + x1;
      py = i + y1;
      draw_point( px, py, v );
    }
  }
}

void fill_rect( u08 left, u08 top, u08 right, u08 bottom, u08 v )
{
  u08           x, y;

  for ( x = left + 1; x < right; x++ )
  {
    for ( y = top + 1; y < bottom; y++ )
      draw_point( x, y, v );
  }
}

void draw_rectangle( u08 left, u08 top, u08 right, u08 bottom, u08 linestate,
    u08 fillstate )
{
  draw_line( left, top, right, top, linestate );
  draw_line( left, top, left, bottom, linestate );
  draw_line( right, top, right, bottom, linestate );
  draw_line( left, bottom, right, bottom, linestate );
  fill_rect( left, top, right, bottom, fillstate );
}

unsigned char *font;
u08 wfont, hfont, charsize;

void select_font( unsigned char *f, u08 w, u08 h )
{
  font = f;
  wfont = w;
  hfont = h;
  charsize = (h+7)/8 * w; 
}

void draw_text( u08 x, u08 y, unsigned char *text , u08 m )
{
  u08 i, tx, ty;
  unsigned char *p;

  ty = y - hfont;
  for( p = text, i = 0; *p; i++, p++ )
  {
    tx = x + i * wfont;
    if( tx + wfont > LCD_COLS ) 
    {
      i = 0;
      tx = x;
      ty += hfont;
    }

    switch(m)
    {
     default:
     case LCD_PIXEL_INV:
      show_icon( font + *p * charsize, wfont, hfont, tx, ty, LCD_PIXEL_INV );
      break;

     case LCD_PIXEL_ON:
      show_icon( font + *p * charsize, wfont, hfont, tx, ty, LCD_PIXEL_ON );
      break;
  
     case LCD_PIXEL_OFF:
      show_icon( font + *p * charsize, wfont, hfont, tx, ty, LCD_PIXEL_OFF ); 
      break;
    }
  }
}
  
